from pod2grid.schema import Message, CategoricalLabel
from pprint import pprint
import json
from pymemri.pod.client import PodClient
from pymemri.data.schema import Account


def create_dummy_data(pod_client: PodClient):
    """Creates 3 dummy data points, one of which is labeled.

    The plugin should only sync the labeled data to Domain.
    """
    assert pod_client.add_to_schema(CategoricalLabel(name="", index=0, names=""))
    assert pod_client.add_to_schema(Message(content="", service=""))

    # Delete old dummy messages
    # old_messages = pod_client.search({"type": "Message", "service": "DummyService"})
    # pod_client.bulk_action(delete_items=old_messages)

    # Generate a few test messages
    contents = ["That's amazing!", "That's terrible", "Hi there", "It's a great day", "It's a horrible day"]

    # Label the first two messages
    messages = [Message(content=content, service="DummyService") for content in contents]
    account = Account(externalId="openmined_domain")

    labels = [
        CategoricalLabel(
            name="positive",
            index=2,
            names=json.dumps(["negative", "neutral", "positive"]),
        ),
        CategoricalLabel(
            name="negative",
            index=0,
            names=json.dumps(["negative", "neutral", "positive"]),
        ),
    ]

    messages[0].add_edge("label", labels[0])
    messages[1].add_edge("label", labels[1])

    edges = messages[0].get_edges("label") + messages[1].get_edges("label")
    print(edges)

    pod_client.bulk_action(create_items=messages + labels + [account])
    pod_client.bulk_action(create_edges=edges)


# def create_dummy_data(pod_client):
#     """Creates 3 dummy data points, one of which is labeled.

#     The plugin should only sync the labeled data to Domain.
#     """
#     assert pod_client.add_to_schema(CategoricalLabel(name="", labels=""))
#     assert pod_client.add_to_schema(Message(content="content", service="DummyService"))

#     schema_name = Message.__name__
#     data_field = "content"

#     # Generate a few test messages
#     contents = ["That's amazing!", "That's terrible", "Hi there"]
#     labels = [2, 0, 1]

#     messages = [Message(content=content) for content in contents]

#     for message in messages:
#         pod_client.create(message)

#     pprint("pod content")    
#     pprint(pod_client.search({"type": schema_name}))

#     # Add single edge
#     sentiment = CategoricalLabel(name="positive", index=0)
#     pod_client.create(sentiment)

#     messages[0].add_edge("label", sentiment)

#     edge = messages[0].get_edges('label')[0]
#     pod_client.create_edges([edge])
