import json
import secrets
from pathlib import Path

from pymemri.plugin.schema import Account
from pymemri.pod.utils import PYMEMRI_FOLDER
from syft.grid.client.client import connect
from syft.grid.client.exceptions import RequestAPIException
from syft.grid.client.grid_connection import GridHTTPConnection
import os, signal
import psutil


CREDENTIALS_FILE = Path.home() / PYMEMRI_FOLDER / "pysyft" / "domain" / "credentials.json"
DOMAIN_URL = f"http://localhost:5000"


def kill_domain(proc):
    process = psutil.Process(proc.pid)
    for cproc in process.children(recursive=True):
        cproc.kill()
    try:
        process.kill()
    except Exception:
        # already killed
        pass

def setup_domain(email, pod_client, url=DOMAIN_URL):
    domain = connect(url=url, conn_type=GridHTTPConnection)

    credentials = {
        "email": email,
        "password": secrets.token_urlsafe(),
        "url": url,
        "token": secrets.token_urlsafe(8),
    }

    try:
        domain = domain.setup(**credentials, domain_name="OpenMined Node")

        account = Account(
            identifier=credentials["email"],
            secret=credentials["password"],
            service=url,
            externalId="openmined_domain",
        )
        pod_client.create(account)

        # CREDENTIALS_FILE.parent.mkdir(parents=True, exist_ok=True)
        # with open(CREDENTIALS_FILE, "w") as f:
        #     json.dump(credentials, f)

    except RequestAPIException as e:
        print(e)
        return


def login_domain(pod_client, url=DOMAIN_URL, account_id=None):
    if account_id:
        account = pod_client.get(account_id)
    else:
        accounts = pod_client.search(
            {"type": "Account", "externalId": "openmined_domain"}
        )
        account = accounts[-1]

    return connect(
        url=url,
        credentials={"email": account.identifier,
                     "password": account.secret},
        conn_type=GridHTTPConnection,
    )
