import json
from multiprocessing import Process
from os import kill
from tkinter import W
from typing import Any, Dict

import syft as sy
from pymemri.plugin.pluginbase import PluginBase

from .data_utils import DataField
from .grid_utils import kill_domain, login_domain, setup_domain
from .schema import CategoricalLabel, Message
import subprocess
from time import sleep
from threading import Thread


class TextSyncPlugin(PluginBase):
    _default_config = {
        "item_type": "Message",
        "item_filters": {},
        "data_fields": {"content": "content", "label": "label.index"},
        "domain_tag": "whatsapp",
    }


    def __init__(self, domain_cwd="./domain", **kwargs):
        super().__init__(**kwargs)
        if self.pluginRun:
            self.account = self.pluginRun.account[0]
        self.set_config()
        self._domain_cwd=domain_cwd
        self._domain_set_up=False

    def set_config(self):
        if self.pluginRun and self.pluginRun.settings:
            config = json.loads(self.pluginRun.settings)
        else:
            config = dict()

        for item in self._default_config:
            value = config.get(item, self._default_config[item])
            setattr(self, item, value)

    def upload_dataset_to_domain(self, dataset_prefixes: str, data: Dict[str, Any]):
        """
        Sends all items (field_name, field_data) in data to Domain with tag "self.domain_tag:field_name".
        Any field data is sendable if syft downcast is available.

        Args:
            domain_client ([type]): grid domain client to send data to.
            data (dict[str, Any]): dict with field names and field data to send to domain.
        """

        for field, field_data in data.items():
            field_data = sy.lib.python.util.downcast(field_data)
            tags = [f"{dataset_prefix}:{field}" for dataset_prefix in dataset_prefixes]
            field_data.send(self.domain_client, pointable=True, tags=tags)

    def sync_with_domain(self, dataset):
        data_dict = dataset.to("dict", columns=["content", 'label.value'], filter_missing=True)
        n_items = len(next(iter(data_dict.values())))
        print(f"Syncing {n_items} items to grid...")
        self.upload_dataset_to_domain([dataset.id, dataset.name], data_dict)
    
    def setup_dataset_syncer(self, poll_interval=1):
        def poll():
            while True:
                datasets = self.client.search({"type": "Dataset"})
                for ds in datasets:
                    self.sync_with_domain(ds)
                sleep(poll_interval)
        thread = Thread(target=poll)
        thread.start()
        return thread
    
    def setup_domain(self):
        setup_domain("user@memri.io", self.client)
        self.domain_client = login_domain(self.client)
        self._domain_set_up=True

    def run(self):
        # TODO where to get user email + domain IP
        if not self._domain_set_up:
            self.setup_domain()
        ds_sync_thread = self.setup_dataset_syncer()

        print("done setting up pod2grid plugin")
        # wait untill thread finishes (never)
        ds_sync_thread.join()


    def add_to_schema(self):
        self.client.add_to_schema(Message, CategoricalLabel)
