import pytest
from pymemri.pod.client import PodClient, DEFAULT_POD_ADDRESS
from pod2grid.plugin import TextSyncPlugin
from pod2grid.dummy_data import create_dummy_data
from pod2grid.grid_utils import kill_domain
from pod2grid.schema import CategoricalLabel
from pymemri.data.dataset import Dataset
from pymemri.data.schema import Message, Label, Item
from time import sleep
from multiprocessing import Process

import os, signal



def create_test_dataset(client, ds_name):
    client.add_to_schema(Dataset, Message, CategoricalLabel)
    ds = Dataset(name=ds_name)
    messages = [Message(content="first message"), Message(content="second message"), Message(content="third message")]
    labels = [CategoricalLabel("positive"), CategoricalLabel("negative"), CategoricalLabel("positive")]

    for m in messages: ds.add_edge("item", m)
    for m, l in zip(messages, labels): m.add_edge("label", l)
    client.bulk_action(create_items=messages+labels+[ds],
                       create_edges=ds.get_all_edges() + [e for m in messages for e in m.get_all_edges() ] )

@pytest.mark.integration
def test_connect_and_upload_dataset():
	# YOU NEED A ***FRESH*** DOMAIN RUNNING FOR THIS TO PASS 

	client = PodClient()
	ds_name = "test dataset"
	create_test_dataset(client, ds_name=ds_name)

	plugin = TextSyncPlugin(client=client)
	plugin.setup_domain()
	store = plugin.domain_client.store
	# syncs to store
	plugin_process = Process(target=plugin.run, daemon=True)
	plugin_process.start()

	sleep(2)

	# Query and run tests
	def get_by_tag(store, tag):
		return [x for x in store if tag in x.tags]

	x = get_by_tag(store, f"{ds_name}:content")[0]
	y = get_by_tag(store, f"{ds_name}:label.value")[0]

	x_res = x.get()
	y_res = y.get()

	assert x_res == ['first message', 'second message', 'third message']
	assert y_res == ['positive', 'negative', 'positive'] 



